#========================================================================================
# package      DE_EPAGES::AmazonPayments::t::AmazonPayments
# base         DE_EPAGES::Shop::API::Test
# state        private
#========================================================================================
package DE_EPAGES::AmazonPayments::t::AmazonPayments;
use base DE_EPAGES::Shop::API::Test;
use strict;

## no critic (ProhibitMagicNumbers ProhibitMultiplePackages)
use Data::Dumper qw(Dumper);
use Test::More;
use Test::Exception;
use DE_EPAGES::Order::API::Basket qw ( CreateBasket );
use DE_EPAGES::Order::API::LineItemContainer qw ( SavePayment );
use DE_EPAGES::AmazonPayments::API::Object::PaymentMethodAmazonPayments;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::AmazonPayments::API::Constants qw(
    AMAZON_REGION_DE
);
use DE_EPAGES::Object::API::Factory qw (
    LoadObjectByPath
    LoadRootObject
    ExistsObjectByPath
);
use DE_EPAGES::Shop::API::Constants qw ( TAX_MODEL_GROSS );

sub init() {
    my $self = shift;
    $self->SUPER::init();

    $self->shop->featureMaxValue('PaymentMethods', 10);
    $self->shop->featureMaxValue('AmazonPayments', 1);
    $self->shop->featureMaxValue('Basket', 1);
    $self->shop->featureMaxValue('Products', 1);

    LoadObjectByPath('/PaymentTypes/Free')->newPaymentMethod($self->shop, {
        IsActivated => 1,
        IsDefault   => 1,
    });
    # defaults to a valid region for AmazonPayments
    $self->shop->set({
        CurrencyID => 'EUR',
        LocaleID  => 'de_DE',
    });
    my $ProductFolder = $self->shop->folder('Products');
    my $Product = $ProductFolder->insertChild('Product', {
        Alias => 'TestProduct',
        Shop  => $self->shop,
    });
    $Product->setListPrice('EUR', 0, 1);
}

sub test {
    my $self = shift;
    $self->testPaymentMethod();
    $self->testLineItem();
    done_testing;
}

sub testPaymentMethod {
    my $self = shift;
    my $PaymentType = MockedPaymentType->new;
    my $PaymentMethod = $PaymentType->newPaymentMethod($self->shop, {
        IsActivated => 1,
        IsDefault => 0,
        CredentialsChecked => 1,
        Alias => 'AmazonPaymentsTest',
    });

    # regions
    my $hRegions = $PaymentMethod->regions;
    is((scalar keys %$hRegions), 3, 'All regions');
    is_deeply([sort keys %{$hRegions->{'DE'}}], [qw(IsActive Label)], 'regions keys') or diag Dumper($hRegions);

    # regionKeys
    $PaymentType->mockAttributes({
        DeveloperKey_DE => 'devkey_de',
        DeveloperKey_UK => undef,
        DeveloperKey_US => 'devkey_us',
        AccessKeyId_DE => 'accesskey_de',
        AccessKeyId_UK => 'accesskey_uk',
        AccessKeyId_US => undef,
        SecretKey_DE => 'secret_de',
        SecretKey_UK => undef,
        SecretKey_US => 'secret_us',
    });

    my $hRegionKeys = $PaymentMethod->regionKeys(AMAZON_REGION_DE);
    is($hRegionKeys->{DeveloperKey}, 'devkey_de', 'DeveloperKey');
    is($hRegionKeys->{AccessKeyId}, 'accesskey_de', 'AccessKeyId');
    is($hRegionKeys->{SecretKey}, 'secret_de', 'SecretKey');

    # regionForSF
    is($PaymentMethod->regionForSF('de_DE', 'EUR'), AMAZON_REGION_DE, 'regionForSF DE');
    is($PaymentMethod->regionForSF('en_GB', 'GBP'), undef, 'regionForSF UK');

    throws_ok(sub { $PaymentMethod->regionKeys('INVALID') }, 'DE_EPAGES::Core::API::Error', 'Invalid region id');

    # usableAtStorefront
    ok(! $PaymentMethod->usableAtStorefront({CurrencyID => 'EUR', TaxModel => TAX_MODEL_GROSS }), 'missing merchant id');
    $PaymentMethod->set({ MerchantID => 'SOME_MERCHANT', AmazonRegion => 'DE' });
    ok($PaymentMethod->usableAtStorefront({CurrencyID => 'EUR', TaxModel => TAX_MODEL_GROSS }), 'usable');

    # canAddToBasket
    ok($PaymentMethod->canAddToBasket({CurrencyID => 'EUR', LocaleID => 'de_DE', TaxModel => TAX_MODEL_GROSS }), 'canAddToBasket ok');
    ok(!$PaymentMethod->canAddToBasket({CurrencyID => 'GBP', LocaleID => 'en_GB', TaxModel => TAX_MODEL_GROSS }), 'canAddToBasket, wrong locale en_GB');
    ok(!$PaymentMethod->canAddToBasket({CurrencyID => 'EUR', LocaleID => 'es_AG', TaxModel => TAX_MODEL_GROSS }), 'canAddToBasket, wrong locale es_AG');
    ok(!$PaymentMethod->canAddToBasket({CurrencyID => 'GBP', LocaleID => 'de_DE', TaxModel => TAX_MODEL_GROSS }), 'canAddToBasket, wrong currency');

    # client
    ok($PaymentMethod->client({Region => 'DE'}), 'can create RestClient');

    # don't allow two instances.
    throws_ok sub {
        $PaymentType->newPaymentMethod($self->shop, {
            IsActivated => 1,
            IsDefault => 1,
            Alias => 'AmazonPaymentsTest2',
        });
    }, qr/NoSecondInstanceAllowed/, 'No Second Instance Allowed';

    $PaymentType->mockAttributes({
        IPNPathTemplate => '/rs/payments/amazon/callback/{shopId}',
    });

    my $System = LoadRootObject;
    my $HasSSL = $System->get('HasSSLCertificate');

    eval {
        $System->set({HasSSLCertificate => 0});
        ok(!$PaymentMethod->callbackUrl(), 'without SSL certificate there is no callback possible.');

        $System->set({HasSSLCertificate => 1 });
        $self->shop->set({'DomainName' => 'somehost.tld', 'WebServerScriptNamePart' => 'MyShop' });
        is($PaymentMethod->callbackUrl(), 'https://'.$System->get('DomainName').'/rs/payments/amazon/callback/MyShop', 'with SSL certificate .');
    };
    if (ExistsError()) {
        fail(GetError->longMessage);
    }

    # reset
    $System->set({HasSSLCertificate => $HasSSL });
}

sub testLineItem {
    my $self = shift;

    # payment method from previous test.
    my $PaymentMethod = $self->shop->child('PaymentMethods')->child('AmazonPaymentsTest');

    my $UserFolder = $self->shop->folder('AnonymousUsers');
    my $BasketUserID = $UserFolder->nextAlias('%d');
    my $BasketUser = $UserFolder->insertChild('AnonymousUser', {
        'Alias' => $BasketUserID,
        'Name'  => $BasketUserID,
    });

    my $Basket = CreateBasket({
        User => $BasketUser,
        Shop => $self->shop,
        LocaleID => 'de_DE',
        CurrencyID => 'EUR'
    });

    $self->shop->path('Products/TestProduct')->addToContainer($Basket->container, { Quantity => 1 });
    SavePayment($Basket->container, $PaymentMethod);
    ok($Basket->container->get('Payment')->instanceOf('LineItemPaymentAmazonPayments'), 'LineItemPaymentAmazonPayments');
}

__PACKAGE__->runScript;

package MockedPaymentType;
use base 'DE_EPAGES::Payment::API::Object::PaymentType';
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );

sub new {
    my $class = shift;
    my $self = LoadObjectByPath('/PaymentTypes/AmazonPayments');
    $self->{MockedAttributes} = {};
    return bless $self, $class;
}

sub mockAttributes {
    my $self = shift;
    my ($hMocked) = @_;
    $self->{MockedAttributes} = {%$hMocked};
    return;
}

sub get {
    my $self = shift;
    my ($aNames) = @_;

    my $hMocked = $self->{MockedAttributes};
    my $Result;
    if(ref($aNames) eq 'ARRAY') {
        $Result = $self->SUPER::get($aNames);
        foreach (keys(%$Result)) {
            $Result->{$_} = $hMocked->{$_} if(exists($hMocked->{$_}));
        }
    }
    else {
        $Result = exists($hMocked->{$aNames}) ? $hMocked->{$aNames} : $self->SUPER::get($aNames);
    }

    return $Result;
}
